@@ -1,4 +1,4 @@
-// $Id: main_thread.c,v 1.21 2006/09/19 09:09:51 khlut Exp $
+// $Id: main_thread.c,v 1.22 2006/09/21 18:20:53 khlut Exp $
 // Copyright 2005 Elphel, Inc.
 //
 // This file is part of GenReS.
@@ -108,12 +108,12 @@ char * objectprop(NPP instance, int object_type, const char *name)
 */
 int do_send_value(PD*pd, char *key, char *value) {
   Debug("%p send %s=%s\n",pd,key,value);
-  return fprintf(pd->slavectrl,"%s=%s\n",key,value);
+  return key? fprintf(pd->slavectrl,"%s=%s\n",key,value) : fprintf(pd->slavectrl,"%s\n",value);
 }
 void send_value(PD *pd, char *key, char *value) //send variable to slave
 {
   PR_Lock(pd->lock);
-  NPIdentifier keyid=NPN.getstringidentifier(key);
+  NPIdentifier keyid=key?NPN.getstringidentifier(key):NULL;
   if(( (!pd->slavectrl) || do_send_value(pd,key,value)<=0 ) && pd->status==Running // try to reanimate slave
      && ( keyid!=run || strcmp(value,"0") )) { 	// ignore run=0 when slave is nonfunctional
     if(debug)perror("write to pipe");
@@ -369,6 +369,8 @@ NPError NPP_New(NPMIMEType mime, NPP instance, uint16 mode, int16 argc,
     pd->stream=0;
     pd->npstream=0;
     pd->autoclose=0;
+    pd->printmode=0;
+    pd->printfile=0;
     
     run_slave(pd,mime,0);	// run slave parallel process
     
@@ -495,7 +497,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* npwindow)
 		XFillRectangle(display, xid, gc, 0, 0, atr.width, atr.height);
 		XSetWindowBackground(display, xid, gcv.foreground);
 		XFreeGC(display, gc);
-		Debug("wsize: %dx%d bg=%d\n", atr.width, atr.height, gcv.foreground);
+		Debug("wsize: %dx%d bg=%lu\n", atr.width, atr.height, gcv.foreground);
 	    }
 	}
     //TODO: display
@@ -524,7 +526,7 @@ NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
       if(reason != NPRES_DONE || Pd->autoclose) {
 	if(Pd->stream)close(Pd->stream);
 	Pd->stream=0;
-      }else {
+      }else{
 	PR_Lock(Pd->lock);
 	if(Pd->slavectrl)fprintf(Pd->slavectrl,"EOF\n");
 	PR_Unlock(Pd->lock);
@@ -552,10 +554,14 @@ void    NPP_Print(NPP instance, NPPrint* printInfo)
 {
     Debug("%p NPP_Print mode=%d\n", instance->pdata, printInfo->mode);
     if(printInfo->mode == NP_EMBED) {
-	FILE *fp=((NPPrintCallbackStruct*)printInfo->print.embedPrint.platformPrint)->fp;
-	if(Pd->slavectrl)fprintf(Pd->slavectrl,"PRINTEMBED\n");
+	process_pending_events(Pd);
+	Pd->printfile=((NPPrintCallbackStruct*)printInfo->print.embedPrint.platformPrint)->fp;
+	send_value(Pd,0,"PRINTEMBED");
+	do {
+	    process_pending_events(Pd);
+	}while(Pd && Pd->printfile);
     }else if(printInfo->mode == NP_FULL) {
-	if(Pd->slavectrl)fprintf(Pd->slavectrl,"PRINTFULL\n");
+	send_value(Pd,0,"PRINTFULL");
 	printInfo->print.fullPrint.pluginPrinted=TRUE;
     }
 }
@@ -617,6 +623,15 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
     char *name= event->str+2;
     NPObject *obj;
     NPVariant val;
+    if(event->pd->printmode) {
+	if(event->pd->printfile) { fputs(name, event->pd->printfile); putc('\n',event->pd->printfile); }
+	if(strcmp(name,"%%EOF")==0) {
+	    event->pd->printfile=0;
+	    event->pd->printmode=0;
+	}
+	Debug("P\n");
+	return 0;
+    }
     if(*name == '{') {
 	STRINGZ_TO_NPVARIANT( name, val );
 	NPN.getvalue(event->pd->instance, NPNVPluginElementNPObject, &obj);
@@ -625,6 +640,13 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 	NPN.releaseobject(obj);
 	return 0;
     }
+    if(strcmp(name,"error=1")==0) {
+	if(event->pd->printfile) {
+	    event->pd->printfile=0;
+	    event->pd->printmode=0;
+	}
+	return 0;
+    }
     char *value= pair(&name, " \t", "=:\n", "\n");
     if(!value || !*name) {
 	if(strcmp(event->str+2,"CLOSE")==0) {
@@ -641,6 +663,14 @@ void * PR_CALLBACK HandleEvent(ObserveEvent* event)
 	    Debug("geturl %s\n",srcv);
 	    NPN.geturl(event->pd->instance, srcv , 0);
 	    free(srcv);
+	}else if(strncmp(event->str+2,"%!PS-",5)==0 && event->pd->printfile) {
+	    event->pd->printmode=1;
+	    fputs(event->str+2, event->pd->printfile); putc('\n',event->pd->printfile);
+	    Debug("PS\n");
+	}else if(strcmp(event->str+2,"%%EOF")==0) {
+	    event->pd->printfile=0;
+	    event->pd->printmode=0; 
+	    Debug("NO-PS\n");
 	}else Debug("I\n");
 	return 0;
     }
